#pragma once

#include <memory>
#include "communication/bridge/client/i_xmlrpc_client.h"
#include "communication/bridge/i_bridge_client.h"
#include "domain/domain.h"
#include <xmlrpc-c/client_simple.hpp>
#include <kr2_program_api/internals/api/console.h>


namespace spin
{

    class BridgeClient : public IBridgeClient
    {
    public:
        BridgeClient(std::shared_ptr<IXmlRpcClient> xmlrpcClient);

        void call(const std::string &method, xmlrpc_c::value *result) const;
        void call(const std::string &method, const xmlrpc_c::paramList &params, xmlrpc_c::value *result) const;
        void call(const std::string &method, const xmlrpc_c::paramList &params, xmlrpc_c::value *result, const std::chrono::milliseconds timeout) const;

        void connect() const override;
        void setHost(const std::string &host) override;
        std::string host() const override;
        MonitoredItems monitoredItems() const override;
        BridgeInfo bridgeInfo() const override;
        ToolInfo toolInfo() const override;
        void enableExhibitionMode() const override;
        std::set<Program> programs() const override;
        std::set<Group> groups() const override;
        void acknowledgeErrors() const override;
        bool isSafetyShieldIdle() const override;
        std::string programMessages() const override;
        double drivingSpeedSetpoint() const override;
        void safetyShieldDoHoming() const override;
        void safetyShieldStop() const override;
        void safetyShieldLockBit() const override;
        void safetyShieldReleaseBit() const override;
        void safetyShieldCoverBit() const override;
        void safetyShieldCoverScrew(const double &length_mm) const override;
        void safetyShieldFollowRobot() const override;
        void safetyShieldMoveTo(const double &length_mm) const override;
        void startMotor(const int speed_rpm, const int rampTime_ms = 500) const override;
        void stopMotor() const override;
        ProgramResult latestResult() const override;
        boost::uuids::uuid latestResultUUID() const override;
        ProgramResult::Evaluation latestResultEvaluation() const override;
        UUID startScrewProgram(const UUID &programUuid) const override;
        bool ping() const;


    private:

        std::shared_ptr<IXmlRpcClient> xmlRpcClient_;
    };

} // namespace spin
